/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.dataio.provider.xmlbeans;

import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerXmlbeans;
import de.unihalle.informatik.Alida.dataio.provider.ALDDataIOXmlbeans;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOManagerException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;
import de.unihalle.informatik.Alida_xml.ALDXMLObjectType;
import java.lang.reflect.Field;

/**
 * Abstract class providing basic methods for xml DataIO using xmlbeans
 * according to Alida conventions.
 *<p>
 * These conventions are detailed in the documentation if the methods <code>readData</code> and <code>writeData</code> in this class below.
 * They handle IO from/to file and reading derived classes of the class handled by an dataIO provider.
 *<p>
 * Classes extending this class are expected to override the methods <code>parse</code> and
 * <code>formatAsString</code> which do the actual reading/parsing or writing/formating
 * subsequent to generic handling of Alida convention with respect to derived classes
 * and IO form/to file.
 *
 * @author posch
 *
 */
public abstract class ALDStandardizedDataIOXmlbeans implements ALDDataIOXmlbeans {

	/**
	 * debugging output
	 */
	private boolean debug = false;


	/** Returns an object instantiated from <code>xmlObject</code>.
	 * For the class of the object to be read see {@link ALDDataIOManagerXmlbeans#readData(Field,Class,ALDXMLObjectType)}.
	 * This method is assumed to directly parse the <code>xmlObject</code> and make no
	 * prior interpretation regarding a file to use or derived class to return.
	 * 
	 *  @see de.unihalle.informatik.Alida.dataio.ALDDataIOManagerXmlbeans
	 *
	 * @param	field Field of object to be returned
	 * @param cl Class of object to be returned.
	 * @param xmlObject   Source from where to read data.
	 * @param object Object instantiated from <code>xmlObject</code> with the default constructor
	 * @return Object with member fields set from <code>xmlObject</code>
	 * @throws ALDDataIOProviderException
	 * @throws ALDDataIOManagerException
	 */
	abstract public Object readData(Field field, Class<?> cl, ALDXMLObjectType xmlObject, Object object)
			throws ALDDataIOProviderException, ALDDataIOManagerException;


	/** Read an object using <code>valueString</code>.
	 * For the class of the object to be read see {@link ALDDataIOXmlbeans}.
	 * <p>
	 *
	 * @param field Field of object to be returned.
	 * @param cl Class of object to be returned.
	 * @param valueString   Source from where to read data (e.g. a filename).
	 * @return      Object read from source.
	 * @throws ALDDataIOProviderException 
	 * @throws ALDDataIOManagerException 
	 *
	 */

	/* (non-Javadoc)
	 * @see de.unihalle.informatik.Alida.helpers.ALDDataIO#writeData(java.lang.Object, java.lang.String)
	 */
	
	@Override
	public Object readData(Field field, Class<?> cl, ALDXMLObjectType xmlObject) 
			throws ALDDataIOProviderException, ALDDataIOManagerException {
			
		if ( field != null )
			cl = field.getType();
		
		if ( xmlObject == null || xmlObject.isNil())
			return null;
		
		Object obj = null;
		if ( ALDXMLObjectType.class.isAssignableFrom(xmlObject.getClass()) ) {
			// try to get class from xml and instantiate an object with this class
			ALDXMLObjectType aldXmlObject = null;
			aldXmlObject = (ALDXMLObjectType)xmlObject;
			if ( debug ) {
				System.out.println("ALDStandardizedDataIOXmlbeans::readData found class <" +
						aldXmlObject.getClassName() + "> in xml-document");
			}
			Class<?> clazz = null;
			try {
				clazz =Class.forName(aldXmlObject.getClassName()); 
			} catch (ClassNotFoundException e) {
				// probably we get into trouble if we can not find the class name
				// but give it a try and invoke readData with obj== null
			}
			try {
				obj = clazz.newInstance();
				if ( cl.isAssignableFrom(aldXmlObject.getClass()) ) {
					throw new ALDDataIOProviderException( ALDDataIOProviderExceptionType.OBJECT_TYPE_ERROR,
							"ALDStandardizedDataIOXmlbeans::readData found object of type <" +
									aldXmlObject.getClassName() + "> found in xmlObject" +
									"\nwhich is not assignable to <" +
									cl.getName() +">");
				}
			} catch (InstantiationException e) {
				// invoke readData with obj== null
			} catch (IllegalAccessException e) {
				// invoke readData with obj== null
			}
		} else {
			// we do not have an ALDXMLObjectType and can not infer type from xml
			if ( debug ) {
				System.out.println("ALDStandardizedDataIOXmlbeans::readData " +
						xmlObject.getClass().getName());
			}			
		}

		return readData(field, cl, xmlObject, obj);
	}
	
}

