/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2013
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev$
 * $Date$
 * $Author$
 * 
 */

package de.unihalle.informatik.Alida.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Set;

import javax.swing.*;
import javax.swing.event.EventListenerList;

import de.unihalle.informatik.Alida.operator.*;
import de.unihalle.informatik.Alida.operator.events.ALDOpParameterChangeEvent;
import de.unihalle.informatik.Alida.operator.events.ALDOpParameterChangeEventListener;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing.ProviderInteractionLevel;
import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerXmlbeans;
import de.unihalle.informatik.Alida.dataio.provider.swing.components.ALDOperatorParameterPanel;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.*;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException.OperatorExceptionType;

/**
 * Frame to configure an operator.
 * 
 * @author Birgit Moeller
 */
public class ALDOperatorConfigurationFrame extends JFrame 
	implements ActionListener, ALDSwingValueChangeListener {

	/**
	 * Local flag for debug output.
	 */
	@SuppressWarnings("unused")
	private boolean debug = false;

	/**
	 * The operator associated with this frame.
	 */
	protected ALDOperator op = null;

	/**
	 * The top level panel of this frame.
	 */
	protected JPanel inputPanel;

	/**
	 * Title string of window.
	 */
	protected String titleString = null;

	/**
	 * Mode for displaying parameters.
	 * <p>
	 * In standard mode only parameters annotated as standard are shown, in
	 * advanced mode all parameters annotated are displayed.
	 */
	protected Parameter.ExpertMode displayMode = Parameter.ExpertMode.STANDARD;

	/**
	 * Last directory visited, initially it's user's home.
	 */
	protected String lastDirectory = System.getProperty("user.home");

	/**
	 * Last selected file.
	 */
	protected File lastFile = new File("operatorParams.xml");

	/**
	 * Level of interaction providers are allowed to perform.
	 */
	protected ProviderInteractionLevel pLevel =
			ProviderInteractionLevel.ALL_ALLOWED;

	/**
	 * Panel where operator parameters are embedded.
	 */
	protected ALDOperatorParameterPanel operatorParameterPanel;

	/**
	 * Label of status bar, changes dynamically.
	 */
	protected JLabel status;

	/**
	 * Ok label to be used on button of Ok message boxes.
	 */
	protected final Object[] okOption = { "OK" };

	/**
	 * Labels to be used on buttons of Yes/No message boxes.
	 */
	protected final Object[] yesnoOption = { "YES", "NO" };

	/**
	 * List of registered event listeners.
	 */
	protected volatile EventListenerList listenerList =	new EventListenerList();

	/**
	 * Radio button for selecting standard configuration mode.
	 */
	protected JRadioButtonMenuItem radioItemStandard;

	/**
	 * Radio button for selecting advanced configuration mode.
	 */
	protected JRadioButtonMenuItem radioItemAdvanced;

	protected JTabbedPane tabPane;
	
	/** 
	 * Constructs a control frame for an operator object.
	 * @param _op Operator to be associated with this frame object.
	 * @throws ALDOperatorException
	 */
	public ALDOperatorConfigurationFrame(ALDOperator _op) 
			throws ALDOperatorException {
		if (_op == null)
			throw new ALDOperatorException(OperatorExceptionType.INSTANTIATION_ERROR,
				"[ALDOperatorConfigurationFrame(op)] no operator given, object null!");
		this.op = _op;
		init();
	}
	
	/** 
	 * Does the main work to instantiate the frame.
	 */
	protected void init() {
		if (this.op == null) {
			JOptionPane.showOptionDialog(
							null,
							"Problems instantiating chosen operator...\n"
									+ "Please check class implementation or contact programmer.",
							"Warning", JOptionPane.DEFAULT_OPTION,
							JOptionPane.WARNING_MESSAGE, null, this.okOption,
							this.okOption[0]);
			return;
		}

		// figure out if selected operator is 'controllable'
//		if (this.op instanceof ALDOperatorControllable) {
//			this.isOpControllable = true;
//		} else {
//			this.isOpControllable = false;
//		}

		if (this.titleString == null)
			this.titleString = "ALDOperatorConfigurationFrame: "+this.op.getName();
		
		// init the operator executer
//		this.executeOperator = new ALDExecuteOperator();

		// init the window
		this.setupWindow();

	}
	
	/**
	 * Construct the frame to configure an operator.
	 */
	protected void setupWindow() {

		// choose initial window size according to desktop, i.e. to 40% of width;
		// if width is larger than 1600, maybe multiple screens are in use,
		// in that case the width is fixed to a maximum of 1600
		// get size of desktop
		int desktopWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
		
		// estimate size according to number of parameters in panel...
		int paramNum = this.op.getParameterNames().size();

		int windowWidth = 800, windowHeight = 500;
		if (desktopWidth > 1600)
			windowWidth = (int)(0.4 * 1600);
		else
			windowWidth = (int)(0.4 * desktopWidth);
		windowHeight = paramNum * 50 < 700 ? 700 : paramNum * 50;
		this.setSize(new Dimension(windowWidth, windowHeight));

		// set up the main panel containing input panel and status bar
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		
		// set up the input panel
		this.inputPanel = new JPanel();
		BoxLayout ylayout = new BoxLayout(this.inputPanel, BoxLayout.Y_AXIS);
		this.inputPanel.setLayout(ylayout);
		
		// init and add tab pane
		this.tabPane = new JTabbedPane();
		this.inputPanel.add(this.tabPane);

		// setup operator parameter configuration panel
		this.operatorParameterPanel = this.setupParamConfigPanel();
		// wrap into scroll pane
		JScrollPane scrollPane = 
				new JScrollPane(this.operatorParameterPanel.getJPanel());
		// add a new tabulator to the window
		this.tabPane.add(scrollPane);
		this.tabPane.setTitleAt(0, "Operator Configuration");

		// add additional tabs
		HashMap<JComponent, String> additionalTabs = 
				this.setupAdditionalTabContents();
		if (additionalTabs != null) {
			Set<JComponent> comps = additionalTabs.keySet();
			for (JComponent jcomp : comps) {
				scrollPane = new JScrollPane(jcomp);
				this.tabPane.add(scrollPane, additionalTabs.get(jcomp));
			}
		}
		
		JPanel buttonPanel = new JPanel();
		// add button panel with additional, context-specific buttons
		buttonPanel.add(this.addContextSpecificButtonPanel());
		// add panel with close button
		buttonPanel.add(this.addCloseButtonPanel());

		// put everything into a split pane
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				this.inputPanel, buttonPanel);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation((int)(windowHeight*3.0/4.0));
		mainPanel.add(splitPane, BorderLayout.CENTER);
		
		// add status bar
		this.status = new JLabel("Status: Ready");
		JPanel statusBar = new JPanel();
		statusBar.setBorder(BorderFactory.createLoweredBevelBorder());
		statusBar.add(this.status);
		mainPanel.add(statusBar, BorderLayout.SOUTH);

		// add pane to this window
		this.add(mainPanel);

		// add a nice menubar
		JMenuBar mainWindowMenu = new JMenuBar();
		JMenu fileM = new JMenu("File");
		JMenuItem itemSave = new JMenuItem("Save Settings");
		itemSave.setActionCommand("fileM_save");
		itemSave.addActionListener(this);
		JMenuItem itemLoad = new JMenuItem("Load Settings");
		itemLoad.setActionCommand("fileM_load");
		itemLoad.addActionListener(this);
		JMenuItem itemQuit = new JMenuItem("Quit");
		itemQuit.setActionCommand("fileM_quit");
		itemQuit.addActionListener(this);
		fileM.add(itemSave);
		fileM.add(itemLoad);
		fileM.add(new JSeparator());
		fileM.add(itemQuit);
		mainWindowMenu.add(fileM);

		JMenu viewM = new JMenu("View");
		ButtonGroup operatorLevelGroup = new ButtonGroup();
		// default
		this.radioItemStandard = new JRadioButtonMenuItem("Standard");
		if (this.displayMode.equals(Parameter.ExpertMode.STANDARD))
			this.radioItemStandard.setSelected(true);
		this.radioItemStandard.setActionCommand("viewM_standard");
		this.radioItemStandard.addActionListener(this);
		this.radioItemAdvanced = new JRadioButtonMenuItem("Advanced");
		if (this.displayMode.equals(Parameter.ExpertMode.ADVANCED))
			this.radioItemAdvanced.setSelected(true);
		this.radioItemAdvanced.setActionCommand("viewM_advanced");
		this.radioItemAdvanced.addActionListener(this);
		operatorLevelGroup.add(this.radioItemStandard);
		operatorLevelGroup.add(this.radioItemAdvanced);
		viewM.add(this.radioItemStandard);
		viewM.add(this.radioItemAdvanced);
		mainWindowMenu.add(viewM);

		JMenu actionM = new JMenu("Actions");
		JMenuItem resetOpItem = new JMenuItem("Reset to Defaults");
		resetOpItem.setActionCommand("actionM_reset");
		resetOpItem.addActionListener(this);
		actionM.add(resetOpItem);
		mainWindowMenu.add(actionM);

		// add additional menu entries
		Collection<JMenu> addMenus = setupAdditionalMenus();
		for (JMenu m : addMenus)
			mainWindowMenu.add(m);
		
		// generate help menu
		JMenu helpM = this.generateHelpMenu();
		mainWindowMenu.add(Box.createHorizontalGlue());
		mainWindowMenu.add(helpM);

		// and go ..
		this.setTitle(this.titleString);
		this.setSize(new Dimension(
											windowWidth,paramNum * 50 < 700 ? 700 : paramNum * 50));
		this.setJMenuBar(mainWindowMenu);

		// check if operator is properly configured
//		try {
//			// TODO should be realized by call of operator function...
//			this.operatorParameterPanel.validateOperatorParameters(true);
			this.fireParameterChangeEvent(new ALDOpParameterChangeEvent(this));
//			if (this.operatorParameterPanel.validateOperatorParameters(true) == null) {
//				// operator is ready for execution, make button yellow
//				this.runButton.setForeground(Color.green);
//			} else {
//				this.runButton.setForeground(Color.red);
//			}
//		} catch (ALDOperatorException e) {
//			this.runButton.setForeground(Color.red);
//		}
	}

	/**
	 * Adds operator parameter configuration panel to input panel.
	 */
	protected ALDOperatorParameterPanel setupParamConfigPanel() {
		ALDOperatorParameterPanel opPanel = new ALDOperatorParameterPanel(this.op,
				this.displayMode, true, this);
//		opPanel.addValueChangeEventListener(this);
		return opPanel;
//		JScrollPane scrollPane = new JScrollPane(opPanel.getJPanel());
//		this.inputPanel.add(scrollPane);
	}
	
	protected HashMap<JComponent, String> setupAdditionalTabContents() {
		return null;
	}
	
	/**
	 * Adds additional menu items.
	 * @return	List of menus to be added to the menubar.
	 */
	protected Collection<JMenu> setupAdditionalMenus() {
		// no additional menus to add
		return new LinkedList<JMenu>();
	}
	
//	public void changeMode(Parameter.ExpertMode handlingMode) {
//		if (handlingMode == Parameter.ExpertMode.STANDARD)
//			this.radioItemStandard.doClick();
//		else
//			this.radioItemAdvanced.doClick();
//	}

	/**
	 * Adds set of control buttons to the input panel.
	 */
	protected JPanel addContextSpecificButtonPanel() {
		return new JPanel();
	}

	/**
	 * Adds set of control buttons to the input panel.
	 */
	protected JPanel addCloseButtonPanel() {

		// init panel
		JPanel runPanel = new JPanel();
		runPanel.setLayout(new GridLayout(2, 1));

		// close button
		JButton quitButton = new JButton("Close");
		quitButton.setActionCommand("close");
		quitButton.addActionListener(this);
		quitButton.setBounds(50, 60, 80, 30);

		// now set up a panel to hold the button
		JPanel controlPanel = new JPanel();
		controlPanel
				.setLayout(new BoxLayout(controlPanel, BoxLayout.LINE_AXIS));
		controlPanel.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
		controlPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		controlPanel.add(Box.createHorizontalGlue());
		controlPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		controlPanel.add(Box.createRigidArea(new Dimension(10, 0)));
		controlPanel.add(quitButton);
		runPanel.add(controlPanel);
		return runPanel;
	}

	/**
	 * Sets up the help menu.
	 * 
	 * @return Generated help menu.
	 */
	protected JMenu generateHelpMenu() {
		JMenu helpM = new JMenu("Help");
		JMenuItem itemHelp = new JMenuItem("Online Help");
		itemHelp.addActionListener(OnlineHelpDisplayer.getHelpActionListener(
				itemHelp, this.op.getClass().getName()));
		JMenuItem itemAbout = new JMenuItem("About Alida");
		itemAbout.setActionCommand("helpM_about");
		itemAbout.addActionListener(this);
		helpM.add(itemHelp);
		helpM.add(itemAbout);
		return helpM;
	}

	/**
	 * Changes the status message to the given string.
	 * 
	 * @param msg
	 *            Message to be shown in statusbar.
	 */
	protected void setStatus(String msg) {
		this.status.setText("Status: " + msg);
	}

	/**
	 * Clean-up on termination.
	 * @return	True if window was closed.
	 */
	public boolean quit() {
		this.setVisible(false);
		return true;
	}
	
	/**
	 * Specify the provider interaction level.
	 * @param pl		Level to be used in program execution.
	 */
	public void setProviderInteractionLevel(ProviderInteractionLevel pl) {
		this.pLevel = pl;
	}
	
	public boolean setParameterLinked(String paramName, 
																		String sourceOp, String sourceParamName) {
		return this.operatorParameterPanel.setParameterLinked(paramName,
																									sourceOp, sourceParamName);
	}
	
	public boolean setParameterNotLinked(String paramName) {
		return this.operatorParameterPanel.setParameterNotLinked(paramName);
	}

	/**
	 * Updates the window according to the status of the operator parameters.
	 * <p>
	 * In the window the parameter label colors are changed according to the 
	 * configuration status of the operator parameters.
	 * 
	 * @param badParams		List of missing required parameters.
	 */
	public void updateParamConfigurationStatus(Collection<String> badParams) {
		this.operatorParameterPanel.updateConfigurationStatus(badParams);
	}
	
	/**
	 * Updates the configuration window with settings of new operator.
	 * <p>
	 * Note that we assume here that the class of the new operator is equal to 
	 * the class of the old operator.
	 * 
	 * @param newOp		New operator instance.
	 */
	protected void updateOperator(ALDOperator newOp) {
		// update operator and the GUI with new parameter values
		this.op = newOp;
		try {
			HashMap<ALDOpParameterDescriptor, String> failedParams = 
					this.operatorParameterPanel.setNewOperator(this.op);
			if (failedParams != null) {
				StringBuffer msg = new StringBuffer();
				Set<ALDOpParameterDescriptor> keys = failedParams
						.keySet();
				for (ALDOpParameterDescriptor descr : keys) {
					msg.append("Updating parameter " + descr.getLabel()
							+ " failed!\n");
					msg.append("--> " + failedParams.get(descr) + "\n");
				}
				JOptionPane.showOptionDialog(null,
						"Problems loading file, not all parameters could be updated!\n"
								+ msg, "Warning",
								JOptionPane.DEFAULT_OPTION,
								JOptionPane.WARNING_MESSAGE, null,
								this.okOption, this.okOption[0]);
			}
		} catch (ALDDataIOProviderException ex) {
			JOptionPane.showOptionDialog(null,
					"Problems loading file, operator does not match!\n"
							+ ex.getCommentString(), "Error",
							JOptionPane.DEFAULT_OPTION,
							JOptionPane.ERROR_MESSAGE, null, this.okOption,
							this.okOption[0]);
		}
	}
	
	/**
	 * Changes the status message to the given string.
	 * 
	 * @param msg
	 *            Message to be shown in statusbar.
	 */
//	protected void setStatus(String msg) {
//		this.status.setText("Status: " + msg);
//	}

	/**
	 * Show the result frame of the operator.
	 */
//	public void showResult() {
//		this.executeOperator.getResultFrame().setVisible(true);
//	}

	/**
	 * Run the operator.
	 * 
	 * @param wait
	 *            True if operator has to wait until finished.
	 * @return false if validation failed.
	 * @throws ALDOperatorException
	 */
//	public boolean runOperator(boolean wait, boolean showResult) throws ALDOperatorException {
//		HashMap<ALDParameterDescriptor, String> failedParams;
//		failedParams = this.operatorParameterPanel
//				.validateOperatorParameters(true);
//		if (failedParams != null) {
//			this.setVisible(true);
//			return false;
//		}
//		this.runButton.setEnabled(false);
//		if (this.isOpControllable) {
//			this.pauseButton.setEnabled(true);
//			this.stopButton.setEnabled(true);
//		}
//		this.threadStatus = OpThreadStatus.THREAD_RUNNING;
//		this.setStatus("Running...");
//		try {
//			this.operatorParameterPanel.setOperatorParameters();
//			if (this.op instanceof ALDOperatorControllable) {
//				// tell the operator the requested step size via event...
//				ALDConfigurationEvent confev = new ALDConfigurationEvent(this);
//				confev.disableStepwiseExecution();
//				// ... send it to operator
//				((ALDOperatorControllable) this.op)
//						.handleALDConfigurationEvent(confev);
//			}
//			this.executeOperator.init(this.op, this, showResult, this.pLevel);
//			Thread t = new Thread(this.executeOperator);
//			this.editorFrame.setToInExecution(this.nodeName, false);
//			t.start();
//			if (wait)
//				t.join();
//			return true;
//		} catch (Exception ex) {
//			System.out.println(ex.getMessage());
//			ex.printStackTrace();
//		}
//		return false;
//	}

	/**
	 * Validate the operator parameters by requesting values from the panels.
	 * 
	 * @return True, if validation is successful, otherwise false.
	 */
//	public boolean validateParameters(boolean deepValidate, boolean allowWarnings) {
//		try {
//			HashMap<ALDParameterDescriptor, String> failedList = this.operatorParameterPanel
//					.validateOperatorParameters(deepValidate);
//			if (failedList != null && allowWarnings) {
//				StringBuffer msg = new StringBuffer();
//				Set<ALDParameterDescriptor> keys = failedList.keySet();
//				for (ALDParameterDescriptor descr : keys) {
//					msg.append("Reading parameter \"");
//					msg.append(descr.getLabel() + " failed!\n");
//					msg.append("--> " + failedList.get(descr) + "\n");
//				}
//				JOptionPane.showOptionDialog(null, msg,
//						"Warning - Parameter Validation Failed!",
//						JOptionPane.DEFAULT_OPTION,
//						JOptionPane.WARNING_MESSAGE, null, this.okOption,
//						this.okOption[0]);
//				return false;
//			}
//			else if (failedList != null) {
//				return false;
//			}
//		} catch (ALDOperatorException ex) {
//			if (allowWarnings)
//				JOptionPane.showOptionDialog(null,
//						"Call of operator's validate() method failed! Reason:\n"
//								+ ex.getCommentString(), "Warning",
//						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
//						null, this.okOption, this.okOption[0]);
//			return false;
//		}
//		return true;
//	}

	@Override
	public void actionPerformed(ActionEvent e) {

		// local variables
		String command = e.getActionCommand();

		// close the frame
		if (   command.equals("frame_close")
				|| command.equals("close") 
				|| command.equals("fileM_quit")) {
			this.setVisible(false);
		}

		// handle menu item commands

		else if (command.equals("fileM_save")) {
			// open file chooser
			JFileChooser getFileDialog = new JFileChooser();
			getFileDialog.setApproveButtonText("Save");
			getFileDialog.setCurrentDirectory(new File(this.lastDirectory));
			getFileDialog.setSelectedFile(this.lastFile);
			getFileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int returnVal = getFileDialog.showOpenDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				// check if file exists already, if so ask what to do
				File file = getFileDialog.getSelectedFile();
				if (file.exists()) {
					if ( JOptionPane.showOptionDialog(null, 
							"File " + file.getAbsolutePath() + " exists, override?",
							"file exists",
							JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE,
							null, null, null) != 0 ) {
						return;
					}
				}

				HashMap<ALDOpParameterDescriptor, String> failedParams = this.operatorParameterPanel
						.setOperatorParameters();
				if (failedParams != null) {
					StringBuffer msg = new StringBuffer();
					Set<ALDOpParameterDescriptor> keys = failedParams
							.keySet();
					for (ALDOpParameterDescriptor descr : keys) {
						msg.append("Loading parameter " + descr.getLabel()
								+ "failed!\n");
						msg.append("--> " + failedParams.get(descr) + "\n");
					}
					JOptionPane.showOptionDialog(null,
							"Loading parameters failed, not all parameters could be loaded!\n"
									+ msg,
							"Warning - Problems loading parameters!",
							JOptionPane.DEFAULT_OPTION,
							JOptionPane.WARNING_MESSAGE, null,
							this.okOption, this.okOption[0]);
				}
				this.lastFile = file;
				try {
					System.out.println("save with xmlbeans");
					ALDDataIOManagerXmlbeans.writeXml(file.getAbsolutePath(), this.op);
				} catch (Exception e1) {
					System.out.println(e1.getMessage());
					e1.printStackTrace();
				
					
					System.out.println("save with xstream");
					this.op.serializeToXmlFile(file.getAbsolutePath());
				}
				this.lastDirectory = file.getAbsolutePath();
			}
		} else if (command.equals("fileM_load")) {
			// open file chooser
			JFileChooser getFileDialog = new JFileChooser();
			getFileDialog.setCurrentDirectory(new File(this.lastDirectory));
			getFileDialog.setSelectedFile(this.lastFile);
			getFileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int returnVal = getFileDialog.showOpenDialog(null);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = getFileDialog.getSelectedFile();
				this.lastDirectory = file.getAbsolutePath();
				this.lastFile = file;
				ALDOperator newOp;
				try {
					System.out.println("load with xmlbeans");
					newOp = (ALDOperator) ALDDataIOManagerXmlbeans.readXml(file.getAbsolutePath(), ALDOperator.class);
				} catch (Exception e1) {
					System.out.println(e1.getMessage());
					
					System.out.println("load with xstream");
					newOp = this.op.deserializeFromXmlFile(file.getAbsolutePath());
				}
				// update the GUI with new parameter values
				this.updateOperator(newOp);
//				try {
//					HashMap<ALDOpParameterDescriptor, String> failedParams = 
//							this.operatorParameterPanel.setNewOperator(this.op);
//					if (failedParams != null) {
//						StringBuffer msg = new StringBuffer();
//						Set<ALDOpParameterDescriptor> keys = failedParams
//								.keySet();
//						for (ALDOpParameterDescriptor descr : keys) {
//							msg.append("Updating parameter " + descr.getLabel()
//									+ " failed!\n");
//							msg.append("--> " + failedParams.get(descr) + "\n");
//						}
//						JOptionPane.showOptionDialog(null,
//								"Problems loading file, not all parameters could be updated!\n"
//										+ msg, "Warning",
//								JOptionPane.DEFAULT_OPTION,
//								JOptionPane.WARNING_MESSAGE, null,
//								this.okOption, this.okOption[0]);
//					}
//				} catch (ALDDataIOProviderException ex) {
//					JOptionPane.showOptionDialog(null,
//							"Problems loading file, operator does not match!\n"
//									+ ex.getCommentString(), "Error",
//							JOptionPane.DEFAULT_OPTION,
//							JOptionPane.ERROR_MESSAGE, null, this.okOption,
//							this.okOption[0]);
//				}
			}
		} else if (command.equals("viewM_standard")) {
			this.operatorParameterPanel
					.changeViewMode(Parameter.ExpertMode.STANDARD);
			this.repaint();
		} else if (command.equals("viewM_advanced")) {
			this.operatorParameterPanel
					.changeViewMode(Parameter.ExpertMode.ADVANCED);
			this.repaint();
		} 
		else if (command.equals("actionM_reset")) {
			try {
//	      this.operatorParameterPanel.resetOperatorParameters();
				ALDOperator newOp = this.op.getClass().newInstance();
				this.updateOperator(newOp);
//      } catch (ALDDataIOProviderException ex) {
//				JOptionPane.showOptionDialog(null,
//						"Reset of parameters failed!\n" + ex.getCommentString(),
//						"Warning - Reset of parameters failed!",
//						JOptionPane.DEFAULT_OPTION,
//						JOptionPane.WARNING_MESSAGE, null,
//						this.okOption, this.okOption[0]);
      } catch (InstantiationException e1) {
      	JOptionPane.showOptionDialog(null,
      			"Reset of parameters failed, could not instantiate operator!\n" 
      					+ e1.getMessage(), "Warning - Reset of parameters failed!",
      			JOptionPane.DEFAULT_OPTION,
      			JOptionPane.WARNING_MESSAGE, null,
      			this.okOption, this.okOption[0]);
      } catch (IllegalAccessException e2) {
      	JOptionPane.showOptionDialog(null,
      			"Reset of parameters failed!\n" + e2.getMessage(),
      			"Warning - Reset of parameters failed!",
      			JOptionPane.DEFAULT_OPTION,
      			JOptionPane.WARNING_MESSAGE, null,
      			this.okOption, this.okOption[0]);
      }
		}
		else if (command.equals("helpM_about")) {
			JOptionPane
					.showOptionDialog(
							null,
							"Alida / MiToBo Graphical Operator Runner - \n"
									+ "Chosen Operator: "
									+ this.op.getName()
									+ "\n"
									+ " @2012 Martin Luther University Halle-Wittenberg",
							"About Alida / MiToBo", JOptionPane.DEFAULT_OPTION,
							JOptionPane.PLAIN_MESSAGE, null, this.okOption,
							this.okOption[0]);
		}
	}

	@Override
	public void handleValueChangeEvent(ALDSwingValueChangeEvent event) {
		this.fireParameterChangeEvent(new ALDOpParameterChangeEvent(this));
		// check if operator is properly configured now
//		try {
//			HashMap<ALDParameterDescriptor, String> failedParams =
//					this.operatorParameterPanel.validateOperatorParameters(false);
//			if (failedParams == null) {
//				// operator is ready for execution, make button green
//				this.runButton.setForeground(Color.green);
//				if (this.editorFrame != null)
//					this.editorFrame.setToConfigured(this.nodeName, true);
//			} 
//			else {
//				// operator is not ready for execution, make button red
//				this.runButton.setForeground(Color.red);
//				if (this.editorFrame != null)
//					this.editorFrame.setToConfigured(this.nodeName, false);
//			}
//		} catch (ALDOperatorException e) {
//			this.runButton.setForeground(Color.red);
//			if (this.editorFrame != null)
//				this.editorFrame.setToConfigured(this.nodeName, false);
//		}
	}

	// TODO comments
//	public void copyConfiguration(ALDOperatorConfigurationFrame controlFrame)
//			throws ALDDataIOException {
//		this.operatorParameterPanel
//				.copyConfiguration(controlFrame.operatorParameterPanel);
//	}
	
	/**
	 * Adds a listener to this reporter.
	 * @param listener		Listener to be added.
	 */
	public void addParameterChangeEventListener(
													ALDOpParameterChangeEventListener listener) {
		this.listenerList.add(ALDOpParameterChangeEventListener.class, 
													listener);
	}

	/**
	 * Removes a listener from this reporter.
	 * @param listener		Listener to be removed.
	 */
	public void removeParameterChangeEventListener(
													ALDOpParameterChangeEventListener listener) {
		this.listenerList.remove(ALDOpParameterChangeEventListener.class, 
														 listener);
	}

	/**
	 * Sends an event to all registered listeners.
	 * @param ev		Event to be send to all listeners.
	 */
	@SuppressWarnings("unused")
  public void fireParameterChangeEvent(ALDOpParameterChangeEvent ev){
		// get list of listeners 
		Object[] listeners = this.listenerList.getListenerList();
		
		/* listeners will always be non-null as getListenerList() is guaranteed 
		 * to return a non-null array... */

		// process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length-2; i>=0; i-=2) {
			if (listeners[i]==ALDOpParameterChangeEventListener.class) {
				// lazily create the event:
				((ALDOpParameterChangeEventListener)listeners[i+1]).
							handleALDParameterChangeEvent(
									new ALDOpParameterChangeEvent(this));
			}
		}
	}
}
